/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

var EXPORTED_SYMBOLS = ["MetacertApi"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

const METACERT_API_URL =
  "https://api.metacert.com/2.0/9ccd4e2a/label/";

Cu.import("resource://metacert/common/common.js");
Cu.import("resource://metacert/label/labelDTO.js");
Cu.import("resource://gre/modules/Services.jsm");

/**
 * The Metacert Label API.
 */
MetacertApi = {
  /* Logger for this object. */
  _logger : null,

  /**
   * Initializes the resource.
   */
  _init : function() {
    this._logger = Metacert.getLogger("MetacertApi");
    this._console = Services.console;
  },

  /**
   * Retrieves the info from the Metacert servers and updates it locally
   * @param callback the callback method to be called when the response is
   * processed
   * @param domainList: Domains must be separated by the pipe character (|)
   * encoded as %7C (no delimiter for a single domain request).
   */
  getInfo : function(aCallback, aDomainList, aCallbackPL, aElement) {
    let that = this;
    let url = METACERT_API_URL + "get_info/" + aDomainList;
    // local callback that will pre-process the server response
    let localCallback = function(aResponse) {
      let result = that._parseGetInfoResponse(aResponse);

      try {
        if (aCallback) {
          aCallback(result, aCallbackPL, aElement);
        }
      }
      catch(e) {
        that._logger.error(e);
      }
    };

    request = this._doCall(url, localCallback);

    return request;
  },

  _parseGetInfoResponse : function (aResponse) {
    let labelDTOsList = null;

    try {
      aResponse = aResponse.replace('"-stat"', '"stat"');
      aResponse = aResponse.replace(new RegExp('"-type"', 'g'), '"type"');
      let jsonObject = JSON.parse(aResponse);
      let stat = jsonObject.response.stat;

      if ("success" == stat) {
        labelDTOsList = [];
        let labelObject;
        let certificate = jsonObject.response.labels.certificate;
        let certificateLength = certificate.length;
        let type;
        let hosts;
        let timestamp = new Date().getTime();

        for (let i = 0; i < certificateLength; i++) {
          type = certificate[i].type;
          hosts = certificate[i].includehosts;
          hostsLength = hosts.length;
          for (let j = 0; j < hostsLength; j++) {
            labelObject = new Metacert.LabelDTO(hosts[j], timestamp, type, timestamp);
            labelDTOsList.push(labelObject);
          }
        }
      }
    } catch (e) {
      this._logger.error("Error parsing GetInfo response: " + e);
    }

    return labelDTOsList;
  },

  /**
   * Retrieves the info from the Metacert servers and updates it locally
   * @param callback the callback method to be called when the response is
   * processed
   * @param domainList: Domains must be separated by the pipe character (|)
   * encoded as %7C (no delimiter for a single domain request).
   */
  getUpdates : function(aCallback, aDateFrom) {
    let that = this;
    let url = METACERT_API_URL + "get_updates/" + aDateFrom;
    // local callback that will pre-process the server response
    let localCallback = function(aResponse) {
      let result = that._parseGetUpdatesResponse(aResponse);

      try {
        if (aCallback) {
          aCallback(result);
        }
      }
      catch(e) {
        that._logger.error(e);
      }
    };

    request = this._doCall(url, localCallback);

    return request;
  },

  _parseGetUpdatesResponse : function (aResponse) {
    let result = null;

    try {
      aResponse = aResponse.replace('"-stat"', '"stat"');
      //Replace all "-type" with "type"
      aResponse = aResponse.replace(new RegExp('"-type"', 'g'), '"type"');
      let jsonObject = JSON.parse(aResponse);
      let stat = jsonObject.response.stat;
      result = {};
      result.labelDTOsList = null;
      result.dateFrom = null;
      //result.dateFrom = Metacert.getUnixTime(new Date());

      if ("success" == stat) {
        result.dateFrom = jsonObject.response.date;
        result.labelDTOsList = [];
        let labelObject;
        let certificate = jsonObject.response.label.certificate;
        let certificateLength = certificate.length;
        let type;
        let hosts;
        let timestamp = new Date().getTime();

        for (let i = 0; i < certificateLength; i++) {
          type = certificate[i].type;
          hosts = certificate[i].includehosts;
          hostsLength = hosts.length;
          for (let j = 0; j < hostsLength; j++) {
            labelObject = new Metacert.LabelDTO(hosts[j], timestamp, type, timestamp);
            result.labelDTOsList.push(labelObject);
          }
        }
      }
    } catch (e) {
      this._logger.error("Error parsing Get Updates response: " + e);
    }

    return result;
  },

  /**
   * Retrieves the feed of labels from the Metacert servers and updates it locally
   * @param callback the callback method to be called when the response is
   *   processed
   * @param aDateFrom: Returns labels created or updated after this date.
   *   Must be a valid UNIX timestamp
   * @param aCreateBefore:Returns labels created before this date.
   *   Using aCreateBefore will ignore label updates, even when used alongside
   *   aDateFrom parameter. Must be a valid UNIX timestamp
   */
  getFeed : function(aCallback, aDateFrom, aCreateBefore) {
    let that = this;
    let url = METACERT_API_URL + "get_feed";

    if (aDateFrom) {
      url = url + "?date_from=:" + aDateFrom ;
    }

    if (aCreateBefore) {
      url = url + "&created_before=:" + aCreateBefore;
    }
    // local callback that will pre-process the server response
    let localCallback = function(aResponse) {
      let result = that._parseGetFeedResponse(aResponse);

      try {
        if (aCallback) {
          aCallback(result);
        }
      }
      catch(e) {
        this._logger.error(e);
      }
    };

    request = this._doCall(url, localCallback);

    return request;
  },

  _parseGetFeedResponse : function (aResponse) {
    let labelDTOsList = null;

    try {
      aResponse = aResponse.replace('"-stat"', '"stat"');
      aResponse = aResponse.replace(new RegExp('"-type"', 'g'), '"type"');
      let jsonObject = JSON.parse(aResponse);
      let stat = jsonObject.response.stat;

      if ("success" == stat) {
        labelDTOsList = [];
        let labelObject;
        let certificate = jsonObject.response.labels.certificate;
        let certificateLength = certificate.length;
        let type;
        let hosts;
        let timestamp = new Date().getTime();

        for (let i = 0; i < certificateLength; i++) {
          type = certificate[i].type;
          hosts = certificate[i].includehosts;
          hostsLength = hosts.length;
          for (let j = 0; j < hostsLength; j++) {
            labelObject = new Metacert.LabelDTO(hosts[j], timestamp, type, timestamp);
            labelDTOsList.push(labelObject);
          }
        }
      }
    } catch (e) {
      this._logger.error("Error parsing GetFeed response: " + e);
    }

    return labelDTOsList;
  },

  /**
   * Performs Metacert API call
   * @param aCallback The callback function
   * @returns request object
   */
  _doCall : function(aUrl, aCallback) {
    let request =
      Cc["@mozilla.org/xmlextras/xmlhttprequest;1"].
        createInstance(Ci.nsIXMLHttpRequest);
    try {
      request.mozBackgroundRequest = true;
      request.open("GET", aUrl, true);
      request.onreadystatechange = function() {
        if (4 != request.readyState) { return; }
        aCallback(request.responseText);
      };
      request.send(null);
    } catch(e) {
      this._logger.error(e);
    }
    return request;
  }

};

/**
 * Constructor.
 */
(function() { this._init(); }).apply(MetacertApi);
