/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

var EXPORTED_SYMBOLS = [];

const Cu = Components.utils;

Cu.import("resource://metacert/common/common.js");
Cu.import("resource://metacert/common/DB/dbCommon.js");
Cu.import("resource://metacert/common/DB/dbConnection.js");
//Cu.import("resource://metacert/common/Util/utilUtilityService.js");

// Database file name
const DB_FILE_NAME = "metacert.sqlite";
const DB_TEMPLATE_FILE_NAME = "metacert.template.sqlite";
// Remember to modify the sqlite template file and add a new record to the
// scheme_migrations table when you change this value.
const SCHEME_VERSION = 1;
// Select migration sql query.
const SELECT_MIGRATION_QUERY =
  "SELECT version FROM scheme_migrations ORDER BY version DESC LIMIT 1;"

/**
 * The DB Service. It holds the connection to the database file.
 */
Metacert.DB.Service = {
  /* Logger */
  _logger : null,
  /* Connection */
  _connection : null,
  /* Database file */
  _dbFile : null,

  /**
    * Returns the database connection.
    * @returns Metacert.DB.Connection object holding the DB connection.
    */
  get connection() {
    return this._connection;
  },

/**
* Initialize the service.
*/
  _init : function() {
    this._logger = Metacert.getLogger("Metacert.DB.Service");
    this._logger.trace("init");

    this._initializeDatabase();
  },

  /**
* Sets up the database file if it doesn't exist and opens the connection.
*/
  _initializeDatabase : function() {
    this._logger.trace("_initializeDatabase");

    let dbSchemeVersion;

    this._dbFile = Metacert.getExtensionDirectory();
    this._dbFile.append(DB_FILE_NAME);

    // check if the file already exists.
    if (!this._dbFile.exists()) {
      let dbCopied =
        Metacert.extractFileFromZip(
          DB_TEMPLATE_FILE_NAME, DB_FILE_NAME);

      if (!dbCopied) {
        // if not, copy the template file.
        let defaultFile =
          Metacert.getDefaultsFolder(Metacert.ExtensionId);

        defaultFile.append(DB_TEMPLATE_FILE_NAME);
        defaultFile.copyTo(this._dbFile.parent, this._dbFile.leafName);
      }
    }

    this._connection = new Metacert.DB.Connection(this._dbFile);

    dbSchemeVersion = this._getDBSchemeVersion();
    if (dbSchemeVersion < SCHEME_VERSION) {
      this._migrateDatabase(dbSchemeVersion);
    }
  },

  /**
* Gets the scheme version from the database.
* @return version the scheme version.
*/
  _getDBSchemeVersion : function() {
    this._logger.trace("_getDBSchemeVersion");

    let version = 0;
    let query = null;

    try {
      let query = this._connection.createStatement(SELECT_MIGRATION_QUERY);
      let resultSet = this._connection.executeQuery(query);

      if (resultSet.hasMoreElements()) {
        version = (resultSet.getNext()).getValueForColumnName("version");
      }
    } catch (e) {
      // XXX: ignore error, since the scheme_migrations table needs to be added.
    } finally {
      if (null != query) {
        query.finalize();
      }
    }

    return version;
  },

  /**
* Migrates database.
* @param aCurrentVersion the current scheme version in the database.
*/
  _migrateDatabase : function(aCurrentVersion) {
    this._logger.trace("_migrateDatabase");

    let sql = null;
    let query = null;

    // update to version 2 scheme.
    if (aCurrentVersion < 2) {

    }
  },

  /**
* Removes the database.
*/
  removeDatabase : function() {
    this._logger.debug("removeDatabase");

    this.connection.closeConnection();
    try {
      this._dbFile.remove(true);
    } catch (e) {
      this._logger.error("removeDatabase. Error:\n" + e);
    }
  },

  /**
   * Executes a non query.
   * @param aSQLQuery the sql query.
   * @param aSQLArguments the sql arguments.
   * @return the query result.
   */
  executeNonQuery : function(aSQLQuery, aSQLArguments) {
    this._logger.debug("executeNonQuery");

    let statement = this.connection.createStatement(aSQLQuery);

    if (aSQLArguments) {
      for (let i = 0; i < aSQLArguments.length; i++) {
        statement.bindUTF8StringParameter(i, aSQLArguments[i]);
      }
    }
    this.connection.executeNonQuery(statement);
  },

  /**
   * Executes a sync query.
   * @param aSQLQuery the sql query.
   * @param aSQLArguments the sql arguments.
   * @return the query result.
   */
  executeQuery : function(aSQLQuery, aSQLArguments) {
    this._logger.debug("executeQuery");

    let resultSet = null;
    let statement = this.connection.createStatement(aSQLQuery);

    if (aSQLArguments) {
      for (let i = 0; i < aSQLArguments.length; i++) {
        statement.bindUTF8StringParameter(i, aSQLArguments[i]);
      }
    }
    resultSet = this.connection.executeQuery(statement);

    return resultSet;
  },

  /**
   * Execute asynchronous sql queries.
   * @param aSQLQuery the sql query.
   * @param aSQLArguments the sql arguments.
   * @param aCallback the callback method.
   * @return the query result.
   */
  executeAsyncQuery : function(aSQLQuery, aSQLArguments, aCallback) {
    this._logger.debug("executeAsyncQuery");

    let result = null;
    let resultRows = new Array();
    let statement = this.connection.createStatement(aSQLQuery);

    if (aSQLArguments) {
      for (let i = 0; i < aSQLArguments.length; i++) {
        statement.bindUTF8StringParameter(i, aSQLArguments[i]);
      }
    }

    var localCallback = function(aRows) {
      if (aCallback) {
        aCallback(aRows);
      }
    }
    result = this.connection.executeAsyncQuery(statement, localCallback);
  }
};

/**
* Constructor
*/
(function() { this._init(); }).apply(Metacert.DB.Service);
