/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

const EXPORTED_SYMBOLS = ["Metacert"];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://metacert/common/log4moz.js");
Cu.import("resource://gre/modules/Services.jsm");

/**
 * Metacert namespace.
 */
if ("undefined" == typeof(Metacert)) {
  Metacert = {
    /* The FUEL Application object. */
    _application : null,
    /* Reference to the observer service. */
    _observerService : null,

    /**
     * Initialize this object.
     */
    _init : function() {
      // The basic formatter will output lines like:
      // DATE/TIME  LoggerName LEVEL  (log message)
      let formatter = new Log4Moz.MetacertFormatter();
      let root = Log4Moz.repository.rootLogger;
      let logFile = this.getExtensionDirectory();
      let app;

      logFile.append("log.txt");

      // Loggers are hierarchical, lowering this log level will affect all
      // output.
      root.level = Log4Moz.Level["All"];

      // A console appender outputs to the JS Error Console.
      // app = new Log4Moz.ConsoleAppender(formatter);
      // app.level = Log4Moz.Level["All"];
      // root.addAppender(app);

      // A dump appender outputs to standard out.
      //app = new Log4Moz.DumpAppender(formatter);
      //app.level = Log4Moz.Level["Warn"];
      //root.addAppender(app);

      // This appender will log to the file system.
      app = new Log4Moz.RotatingFileAppender(logFile, formatter);
      app.level = Log4Moz.Level["Fatal"];
      root.addAppender(app);

      // get the observer service.
      this._observerService =
        Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
    },

    /**
     * Gets a logger repository from Log4Moz.
     * @param aName the name of the logger to create.
     * @param aLevel (optional) the logger level.
     * @return the generated logger.
     */
    getLogger : function(aName, aLevel) {
      let logger = Log4Moz.repository.getLogger(aName);

      logger.level = Log4Moz.Level[(aLevel ? aLevel : "All")];

      return logger;
    },

    /**
     * Gets the FUEL Application object.
     * @return the FUEL Application object.
     */
    get Application() {
      // use lazy initialization because the FUEL object is only available for
      // Firefox and won't work on XUL Runner builds.

      if (null == this._application) {
        try {
          this._application =
            Cc["@mozilla.org/fuel/application;1"].
              getService(Ci.fuelIApplication);
        } catch (e) {
          throw "The FUEL application object is not available.";
        }
      }

      return this._application;
    },

    /**
     * Gets the preference branch.
     * @return the preference branch.
     */
    get PrefBranch() { return "extensions.metacert."; },

    /**
     * Gets the id of this extension.
     * @return the id of this extension.
     */
    get ExtensionId() { return "{4A627709-9DBB-44B0-A02C-BE049AE901AA}"; },

    /**
     * Gets the observer service.
     * @return the observer service.
     */
    get ObserverService() { return this._observerService; },

    /**
     * Gets a reference to the directory where the extension will keep its
     * files. The directory is created if it doesn't exist.
     * @return reference (nsIFile) to the extension directory.
     */
    getExtensionDirectory : function() {
      // XXX: there's no logging here because the logger initialization depends
      // on this method.

      let directoryService = Services.dirsvc;
      let profDir = directoryService.get("ProfD", Ci.nsIFile);

      profDir.append("Metacert");

      if (!profDir.exists() || !profDir.isDirectory()) {
        // read and write permissions to owner and group, read-only for others.
        profDir.create(Ci.nsIFile.DIRECTORY_TYPE, 0774);
      }

      return profDir;
    },

   /**
    * Verify if the extension is running over Firefox 4.
    * @return true if Firefox 4, false otherwise.
    */
    get isFirefox4() {
      let isFirefox4 = true;

      if ("@mozilla.org/extensions/manager;1" in Cc) {
        isFirefox4 = false;
      }

      return isFirefox4;
    },

   /**
    * Extracts file from the extension zip.
    * @param aFileName the file name to be extracted.
    * @param aFileNameTo the file name to be used when extracted.
    * @return result true if success, false otherwise.
    */
    extractFileFromZip : function(aFileName, aFileNameTo) {
      let directoryService = Services.dirsvc;
      let zipFile = directoryService.get("ProfD", Ci.nsIFile);
      let result = false;

      zipFile.append("extensions");
      zipFile.append(this.ExtensionId + ".xpi");

      if (zipFile.exists()) {
        let zipReader =
          Cc["@mozilla.org/libjar/zip-reader;1"].getService(Ci.nsIZipReader);
        let fileTo = directoryService.get("ProfD", Ci.nsIFile);
        let filePath = "defaults/" + aFileName;

        zipReader.open(zipFile);
        zipReader.test(null);

        // extract the file
        fileTo.append("Metacert");
        //fileTo.append("BlackFiles");
        fileTo.append(aFileNameTo);
        if (!fileTo.exists()) {
          fileTo.create(Ci.nsIFile.NORMAL_FILE_TYPE, 0777);
          if (zipReader.hasEntry(filePath)) {
            zipReader.extract(filePath, fileTo);
          }
        }

        zipReader.close();
        result = true;
      }

      return result;
    },

   /**
    * Returns an array of elements that match the given aXPathExpression,
    * from the aXMLNode specified, using xpath.
    * @param aXMLNode the XML node or document to search for matches.
    * @param aXPathExpresion the xpath expression to be used in the search.
    * @return array with the elements that matched the xpath expression.
    */
    evaluateXPath : function (aXMLNode, aXPathExpression) {
      let matches = [];
      let xpe =
        Cc["@mozilla.org/dom/xpath-evaluator;1"].
          getService(Ci.nsIDOMXPathEvaluator);

      let nsResolver =
        xpe.createNSResolver(
          ((null == aXMLNode.ownerDocument) ? aXMLNode.documentElement :
           aXMLNode.ownerDocument.documentElement));
      let result = xpe.evaluate(aXPathExpression, aXMLNode, nsResolver, 0, null);
      let res = result.iterateNext();

      while (res) {
        matches.push(res);
        res = result.iterateNext();
      }

      return matches;
    },

    //Format number: if it is from 0-9, it adds a zero
    _format : function (val) {
      val = String(val);
      if (val.length < 2) {
        val = "0" + val;
      }
      return val;
    },

    //"YYYY-MM-DDTHH:MM:SS"  d = new Date();
    //@param d = Date to UTC format
    getDateUTCFormat : function (d){
      let utcDate = d.getUTCFullYear() + "-" + this._format(d.getUTCMonth() + 1)
        + "-" + this._format(d.getUTCDate()) + "T" +
          this._format(d.getUTCHours()) + ":" +
            this._format(d.getUTCMinutes()) +  ":" +
              this._format(d.getUTCSeconds());
      return utcDate;
    },

     //@param aDate = Date to convert to unix time format
    getUnixTime : function (aDate) {
      let milliseconds = aDate.getTime();
      let unixtime = parseInt(milliseconds / 1000);

      return unixtime;
    },


    /**
     * @param aCallback the callback method to be called with the version value
     * returned by the respective component
     */
    getVersion : function(aCallback) {
      Cu.import("resource://gre/modules/AddonManager.jsm");
      AddonManager.getAddonByID(Metacert.ExtensionId, function(aAddon) {
        aCallback(aAddon.version);
      });
    },

    /**
     * Returns true if the given string is null or empty.
     * @param aString the string to check.
     * @return true if the string is null or empty. Returns false otherwise. Note
     * that this method returns false for strings with blank characters, like " ".
     */
    isNullOrEmpty : function(aString) {
      this._logger.debug("isNullOrEmpty");

      return ((null == aString) || (0 == aString.length));
    },

    /**
     * Obtains a reference to the defaults folder for a specific extension.
     * @param aExtensionID the ID of the extension.
     * @return nsIFile reference to the defaults folder for the extension matching
     * the given ID.
     */
   getDefaultsFolder : function(aExtensionID) {
     this._logger.debug("getDefaultsFolder. ID: " + aExtensionID);

     let extensionManager =
       Cc["@mozilla.org/extensions/manager;1"].
         getService(Ci.nsIExtensionManager);
     let defaultsFolder =
       extensionManager.getInstallLocation(aExtensionID).
         getItemLocation(aExtensionID);

     defaultsFolder.append("defaults");

     return defaultsFolder;
   }

  };

  /**
   * Constructor.
   */
  (function() {
    this._init();
  }).apply(Metacert);
}
