/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

const EXPORTED_SYMBOLS = [];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://metacert/common/common.js");
Cu.import("resource://gre/modules/Services.jsm");

// The extension manager action requested observer topic
const TOPIC_ACTION_REQUESTED = "em-action-requested";
// The quit application observer topic
const TOPIC_QUIT_APPLICATION = "quit-application";

/**
 * The Uninstall Service.
 */
Metacert.UninstallService = {
  /* Logger for this object. */
  _logger : null,

  /* Flag indicates whether the uninstall should be executed on exit. */
  _shouldUninstall : false,

  /**
   * Initializes the resource.
   */
  _init : function() {
    this._logger = Metacert.getLogger("Metacert.UninstallService");
    this._logger.trace("_init");

    Metacert.ObserverService.addObserver(this, TOPIC_QUIT_APPLICATION, false);
    Metacert.ObserverService.addObserver(this, TOPIC_ACTION_REQUESTED, false);

    // Firefox 4 requires an special listener to catch add-on related activity
    // (uninstall event mostly). If we try importing the module and there's an
    // exception, then we are not on Firefox 4
    try {
      Cu.import("resource://gre/modules/AddonManager.jsm");
      AddonManager.addAddonListener(this);
    } catch (ex) {}
  },

  /**
   * Cleans up code to remove the extension directory and related preferences.
   */
  _cleanUp : function() {
    this._logger.trace("_cleanUp");

    let prefBranch = Services.prefs;
    let installFolder;

    // remove preferences.
    prefBranch.deleteBranch(Metacert.PrefBranch);

    // close the database connection.
    //Metacert.DB.Service.connection.closeConnection();

    // remove the extension directory if possible.
    try {
      installFolder = Metacert.getExtensionDirectory();
      if (installFolder.exists() && installFolder.isDirectory()) {
        installFolder.remove(true);
      }
    } catch (e) {
      // using debug instead of error because we know that the
      // sqlite file is not going to be removed if the connection is open.
      this._logger.error("_cleanUp. Error:\n" + e);

      // XXX: failed to remove directory, so lets remove contents recursively.
      if (installFolder.exists()) {
        let directoryEntries = installFolder.directoryEntries;
        let entry = null;

        while (directoryEntries.hasMoreElements()) {
          entry = directoryEntries.getNext();
          entry.QueryInterface(Ci.nsIFile);

          try {
            entry.remove(true);
          } catch (e) {
            // if a remove failed, proceed with the next file.
            // XXX: using debug instead of error because we know that the sqlite
            // file is not going to be removed if the connection is open.
            this._logger.error("_cleanUp. Error:\n" + e);
          }
        }
        // try one last time to remove the default folder after we deleted its
        // contents.
        try {
          installFolder.remove(true);
        } catch (e) {
          // XXX: using debug instead of error because we know that the sqlite
          // file is not going to be removed if the connection is open.
          this._logger.error("_cleanUp. Error:\n" + e);
        }
      }
    }
  },

  /**
   * Handles the on uninstalling observer.
   * @param aItem the item to be uninstalled.
   */
  onUninstalling : function(aItem) {
    if (Metacert.ExtensionId == aItem.id) {
      this._shouldUninstall = true;
    }
  },

  /**
   * Handles the on operation cancelled observer.
   * @param aItem the item cancelled.
   */
  onOperationCancelled : function(aItem) {
    if (Metacert.ExtensionId == aItem.id) {
      this._shouldUninstall = false;
    }
  },

  /**
   * Observes global topic changes.
   * @param aSubject the object that experienced the change.
   * @param aTopic the topic being observed.
   * @param aData the data relating to the change.
   */
  observe : function(aSubject, aTopic, aData) {
    this._logger.trace("observe");

    switch(aTopic) {
      case TOPIC_QUIT_APPLICATION:
        if (this._shouldUninstall) {
          this._cleanUp();
        }
        break;
      case TOPIC_ACTION_REQUESTED:
        aSubject.QueryInterface(Ci.nsIUpdateItem);

        if (Metacert.ExtensionId == aSubject.id) {
          switch(aData) {
            case "item-cancel-action":
              if (this._shouldUninstall) {
                this._shouldUninstall = false;
              }
              break;
            case "item-uninstalled":
              this._shouldUninstall = true;
              break;
          }
        }
        break;
    }
  }
};

/**
 * Constructor.
 */
(function() {
  this._init();
}).apply(Metacert.UninstallService);
