/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

var EXPORTED_SYMBOLS = [];

const Cu = Components.utils;

Cu.import("resource://metacert/common/common.js");

/**
 * Object Helper.
 */
Metacert.ObjectHelper = {
  /* Logger for this object. */
  _logger : null,

  /**
   * Initializes the resource.
   */
  _init : function() {
    this._logger = Metacert.getLogger("Metacert.ObjectHelper");
    this._logger.trace("_init");
  },

  /**
   * Gets the typed value.
   * @param aType the type of the value.
   * @param aValue the value.
   * @return the typed value.
   */
  getTypedValue : function(aType, aValue) {
    this._logger.debug("getTypedValue");

    var value;

    if (undefined == aValue || null == aValue) {
      value = null;
    } else {
      switch (aType) {
        case "number":
          if (isNaN(aValue)) {
            value = null;
          } else {
            value = Number(aValue);
          }
          break;
        case "boolean":
          switch (typeof(aValue)) {
            case "boolean":
              value = Boolean(aValue);
              break;
            case "string":
              if ("true" == aValue) {
                value = true;
              } else if ("false" == aValue) {
                value = false;
              } else {
                value = false;
              }
              break;
            default:
              value = false;
              break;
          }
          break;
        case "string":
          value = String(aValue);
          break;
        case "object":
          value = aValue;
          break;
        default:
          value = aValue;
          break;
      }
    }

    return value;
  },

  /**
   * Creates a copy of the param object.
   * @param aObject the object to be copied.
   * @return the object copy.
   */
  copyObject : function(aObject, aSchema) {
    this._logger.debug("copyObject");

    var object = this._getObjectFromName(aSchema.NAME);

    $.each(aSchema.PROPERTIES, function(aName, aValue) {
      object.set(aName, aObject.get(aName));
    });

    return object;
  },

  /**
   * Updates one DTO object with a JSON object.
   * @param aObject the DTO.
   * @param aJSON the JSON.
   * @param aScheme the DTO scheme.
   */
  updateWithJSON : function(aObject, aJSON, aSchema) {
    this._logger.debug("updateWithJSON");

    $.each(aSchema.PROPERTIES, function(aName) {
      aObject.set(aName, aJSON["_" + aName]);
    });
  },

  /**
   * Populates an object from a database row.
   * @param aSchema the object schema.
   * @param aRow the database row.
   * @return the populated object.
   */
  fromRow : function(aSchema, aRow) {
    this._logger.debug("fromRow");

    var object = this._getObjectFromName(aSchema.NAME);

    for (let property in aSchema.PROPERTIES) {
      object.set(property, aRow.getResultByName(property));
    }

    return object;
  },

  /**
   * Populates an object from a XML.
   * @param aXMLNode the xml node.
   * @param aSchema the object schema.
   * @return the populated object from the xml.
   */
  fromXML : function(aXMLNode, aSchema, aExclusivePrefix) {
    this._logger.debug("fromXML");

    var object = this._getObjectFromName(aSchema.NAME);
    var xmlName = null;
    var xmlValue = null;
    var that = this;

    $.each(aSchema.PROPERTIES, function(aName, aInfo) {
      xmlName = null;
      if (aInfo.xml != "") {
        if (aExclusivePrefix) {
          if (aInfo.xml.indexOf(aExclusivePrefix) == 0) {
            xmlName = aInfo.xml.substring(aExclusivePrefix.length+1);
          }
        } else {
          xmlName = aInfo.xml;
        }

        if (xmlName != null) {
          xmlValue = that._getXMLValue(aXMLNode, xmlName);
          if (xmlValue && xmlValue != null && xmlValue != "") {
            object.set(aName, xmlValue);
          } else {
            object.set(aName, aInfo.defaultValue);
          }
        }
      }
    });

    return object;
  },

  /**
   * Gets a value from an XML node, given its name(s) separated by '|'.
   * It allows nested tags, using '.' as a separator and @ for attributes.
   * @param aXMLTree the XML node to traverse.
   * @param aXMLName the tag name to find
   */
  _getXMLValue : function(aXMLTree, aXMLName) {
    this._logger.debug("_getXMLValue");

    var xmlNames = aXMLName.split("|");
    var xmlValue = null;

    $.each(xmlNames, function(aNameIndex, aName) {
      var xmlFields = aName.split(".");
      var xmlNode = aXMLTree;

      $.each(xmlFields, function(aFieldIndex, aField) {
        if (aField[0] != "@") {
          // Check if field name contains an array notation
          var arrayStart = aField.indexOf('[');
          if (arrayStart != -1) {
            var arrayIndex = aField.substring(arrayStart+1, aField.indexOf(']'));
            aField = aField.substring(0,arrayStart);
            xmlNode = $(xmlNode).children(aField).eq(arrayIndex);
          } else {
            xmlNode = $(xmlNode).children(aField);
          }
          if (xmlNode.length == 0) {
            return false;
          }
        } else {
          xmlNode = $(xmlNode);
        }
      });
      if (xmlNode.length > 0) {
        if (xmlFields[(xmlFields.length -1)][0] == "@") {
          var xmlAttr = xmlFields[(xmlFields.length -1)].substring(1);
          xmlValue = $(xmlNode).attr(xmlAttr);
        } else {
          xmlValue = $(xmlNode).text();
        }
        return false;
      }
    });

    return xmlValue;
  },

  /**
   * Gets an object from the name.
   * @param aName the name of the object.
   * @return the object.
   */
  _getObjectFromName : function(aName) {
    this._logger.trace("_getObjectFromName");

    var object;

    switch (aName.toLowerCase()) {
      case "label":
        object = new Metacert.LabelDTO();
        break;
      default:
        this._logger.error(
          "ObjectHelper._getObjectFromName: Object name not found.");
        break;
    }

    return object;
  },

  /**
   * Converts to XML an object.
   * @param aObject the object to be converted.
   * @param aSchema the object schema.
   * @return the xml of the object.
   */
  toXML : function(aObject, aSchema) {
    this._logger.debug("toXML");

    var xml = "<" + aSchema.NAME + ">";

    $.each(aSchema.PROPERTIES, function(aName) {
      if (aObject.get(aName)) {
        xml += "<" + aName + ">" + aObject.get(aName) + "</" + aName + ">";
      }
    });
    xml += "</" + aSchema.NAME + ">";

    return xml;
  }
};

/**
 * Constructor.
 */
(function() { this._init(); }).apply(Metacert.ObjectHelper);
