/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

var EXPORTED_SYMBOLS = [];

const Cu = Components.utils;

Cu.import("resource://metacert/common/common.js");

/**
 * SQL Query Helper.
 */
Metacert.QueryHelper = {
  /* Logger for this object. */
  _logger : null,

  /* Query types. */
  get TYPE_INSERT() { return 1; },
  get TYPE_UPDATE() { return 2; },
  get TYPE_DELETE() { return 3; },
  get TYPE_SELECT() { return 4; },
  get TYPE_SELECT_ALL() { return 5; },
  get TYPE_CREATE_TABLE() { return 6; },
  get TYPE_COUNT_TABLE() { return 7; },
  get TYPE_CLEAR_TABLE() { return 8; },
  get TYPE_STRICT_UPDATE() { return 9;},

  /**
   * Initializes the resource.
   */
  _init : function() {
    this._logger = Metacert.getLogger("Metacert.QueryHelper");
    this._logger.trace("_init");
  },

  /**
   * Builds a SQL query based on parameters.
   * @param aQueryType the query type.
   * @param aTableSchema the table schema.
   */
  buildQuery : function(aQueryType, aTableSchema) {
    this._logger.debug("buildQuery");

    var query;

    switch (aQueryType) {
      case this.TYPE_INSERT:
      case this.TYPE_UPDATE:
        query = "INSERT OR REPLACE INTO '" + aTableSchema.TABLE_NAME + "'(";
        query += this._getColumns(aTableSchema);
        query += ") VALUES (";
        query += this._getColumnsPlaceholders(aTableSchema);
        query += ");";
        break;
      case this.TYPE_STRICT_UPDATE:
        query = "UPDATE '" + aTableSchema.TABLE_NAME + "' SET ";
        query += this._getColumnsWithPlaceholders(aTableSchema);
        query += "WHERE ";
        query += this._getPrimaryKeysWithPlaceholders(aTableSchema);
        query += ";";
        break;
      case this.TYPE_DELETE:
        query = "DELETE FROM '" + aTableSchema.TABLE_NAME + "' WHERE ";
        query += this._getPrimaryKeysWithPlaceholders(aTableSchema);
        query += ";";
        break;
      case this.TYPE_SELECT:
        query = "SELECT ";
        query += this._getColumns(aTableSchema);
        query += " FROM '" + aTableSchema.TABLE_NAME + "' WHERE ";
        query += this._getPrimaryKeysWithPlaceholders(aTableSchema);
        query += ";";
        break;
      case this.TYPE_SELECT_ALL:
        query = "SELECT ";
        query += this._getColumns(aTableSchema);
        query += " FROM '" + aTableSchema.TABLE_NAME + "';";
        break;
      case this.TYPE_CREATE_TABLE:
        query = "CREATE TABLE IF NOT EXISTS '" + aTableSchema.TABLE_NAME + "'(";
        query += this._getColumns(aTableSchema);
        query += ", PRIMARY KEY (";
        query += this._getPrimaryKeys(aTableSchema);
        query += "));";
        break;
      case this.TYPE_COUNT_TABLE:
        query = "SELECT COUNT(*) FROM '" + aTableSchema.TABLE_NAME + "';";
        break;
      case this.TYPE_CLEAR_TABLE:
        query = "DELETE FROM '" + aTableSchema.TABLE_NAME + "';";
        break;
    }

    return query;
  },

  /**
   * Build the array arguments of an object.
   * @param aObject the object.
   * @param aTableSchema the table schema.
   * @return the array arguments of the object.
   */
  buildArguments : function(aObject, aTableSchema) {
    this._logger.debug("buildArguments");

    var arguments = new Array();
    var column = null;

    for (var i = 0; i < aTableSchema.COLUMNS.length; i++) {
      column = aTableSchema.COLUMNS[i];

      if (aObject.get(column) != undefined) {
        arguments.push(aObject.get(column));
      } else {
        arguments.push(null);
      }
    }

    return arguments;
  },

  /**
   * Gets the columns string comma separated.
   * @param aTableSchema the table schema.
   * @return the columns string comma separated.
   */
  _getColumns : function(aTableSchema) {
    this._logger.trace("_getColumns");

    var columns = "";

    for (var i = 0; i < aTableSchema.COLUMNS.length; i++) {
      columns += aTableSchema.COLUMNS[i];
      if (i != aTableSchema.COLUMNS.length - 1) {
        columns += ", ";
      }
    }

    return columns;
  },

  /**
   * Gets the columns placeholders string comma separated.
   * @param aTableSchema the table schema.
   * @return the columns placeholders string comma separated.
   */
  _getColumnsPlaceholders : function(aTableSchema) {
    this._logger.trace("_getColumnsPlaceholders");

    var columns = "";

    for (var i = 0; i < aTableSchema.COLUMNS.length; i++) {
      columns += "?" + (i + 1);
      if (i != aTableSchema.COLUMNS.length - 1) {
        columns += ",";
      }
    }

    return columns;
  },

  /**
   * Gets the columns with placeholders string.
   * @param aTableSchema the table schema.
   * @return the columns with placeholders string.
   */
  _getColumnsWithPlaceholders : function(aTableSchema) {
    this._logger.trace("_getColumnsWithPlaceholders");

    var columns = "";

    for (var i = 0; i < aTableSchema.COLUMNS.length; i++) {
      columns += aTableSchema.COLUMNS[i] + "= ?" + (i + 1);
      if (i != aTableSchema.COLUMNS.length - 1) {
        columns += ", ";
      }
    }

    return columns;
  },

  /**
   * Gets the primary keys string comma separated.
   * @param aTableSchema the table schema.
   * @return the primary keys string comma separated..
   */
  _getPrimaryKeys : function(aTableSchema) {
    this._logger.trace("_getPrimaryKeys");

    var primaryKeys = "";

    for (var i = 0; i < aTableSchema.PRIMARY_KEYS.length; i++) {
      primaryKeys += aTableSchema.PRIMARY_KEYS[i];
      if (i != aTableSchema.PRIMARY_KEYS.length - 1) {
        primaryKeys += ", ";
      }
    }

    return primaryKeys;
  },

  /**
   * Gets the primary keys with placeholders string.
   * @param aTableSchema the table schema.
   * @return the primary keys with placeholders string.
   */
  _getPrimaryKeysWithPlaceholders : function(aTableSchema) {
    this._logger.trace("_getPrimaryKeysWithPlaceholders");

    var primaryKeys = "";

    for (var i = 0; i < aTableSchema.PRIMARY_KEYS.length; i++) {
      primaryKeys += aTableSchema.PRIMARY_KEYS[i] + " = ?" + (i + 1);
      if (i != aTableSchema.PRIMARY_KEYS.length - 1) {
        primaryKeys += " AND ";
      }
    }

    return primaryKeys;
  }
};

/**
 * Constructor.
 */
(function() { this._init(); }).apply(Metacert.QueryHelper);
