/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

var EXPORTED_SYMBOLS = [];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://metacert/common/common.js");
Cu.import("resource://metacert/common/DB/dbService.js");
Cu.import("resource://metacert/helpers/queryHelper.js");
Cu.import("resource://metacert/helpers/objectHelper.js");
Cu.import("resource://metacert/label/labelDTO.js");
Cu.import("resource://gre/modules/Services.jsm");

/**
 * The Label DAO.
 */
Metacert.LabelDAO = {
  /* Table schema. */
  TABLE_SCHEMA : {
    TABLE_NAME : "label",
    COLUMNS : [ "host", "lastUpdate", "certificate", "lastRequest" ],
    PRIMARY_KEYS : [ "host" ]
  },

  /* Logger for this object. */
  _logger : null,

  /**
   * Initialize the resource.
   */
  _init : function() {
    this._logger = Metacert.getLogger("Metacert.LabelDAO");
    this._logger.debug("init");
    this._console = Services.console;
  },

  /**
   * Inserts a new label to the table.
   * @param aLabel LabelDTO with the label data.
   * @param aCallback the callback method.
   */
  insertLabel : function(aLabel, aCallback) {
    this._logger.debug("LabelDAO.insertLabel");

    if ((null == aLabel) || !(aLabel instanceof Metacert.LabelDTO)) {
      throw "Invalid Label DTO.";
    }

    let query =
      Metacert.QueryHelper.buildQuery(
        Metacert.QueryHelper.TYPE_INSERT, this.TABLE_SCHEMA);
    let arguments =
      Metacert.QueryHelper.buildArguments(aLabel, this.TABLE_SCHEMA);

    Metacert.DB.Service.executeAsyncQuery(query, arguments, aCallback);
  },

  /**
   * Updates the label in the table. Works as INSERT OR REPLACE
   * @param aLabel LabelDTO with the label data.
   * @param aCallback the callback method.
   */
  updateLabel : function(aLabel, aCallback) {
    this._logger.debug("LabelDAO.updateLabel");

    if ((null == aLabel) || !(aLabel instanceof Metacert.LabelDTO)) {
      throw "Invalid label DTO.";
    }

    let query =
      Metacert.QueryHelper.buildQuery(
        Metacert.QueryHelper.TYPE_UPDATE, this.TABLE_SCHEMA);
    let arguments =
      Metacert.QueryHelper.buildArguments(aLabel, this.TABLE_SCHEMA);

    Metacert.DB.Service.executeAsyncQuery(query, arguments, aCallback);
  },

  /**
   * Updates the label in the table.
   * @param aLabel LabelDTO with the label data.
   * @param aCallback the callback method.
   */
  strictUpdateLabel : function(aLabel, aCallback) {
    this._logger.debug("LabelDAO.strictUpdateLabel");

    if ((null == aLabel) || !(aLabel instanceof Metacert.LabelDTO)) {
      throw "Invalid label DTO.";
    }

    let query =
      Metacert.QueryHelper.buildQuery(
        Metacert.QueryHelper.TYPE_STRICT_UPDATE, this.TABLE_SCHEMA);
    let arguments =
      Metacert.QueryHelper.buildArguments(aLabel, this.TABLE_SCHEMA);

    Metacert.DB.Service.executeAsyncQuery(query, arguments, aCallback);
  },

  /**
   * Deletes the label from the table.
   * @param aLabelHost the host of the label to delete.
   */
  deleteLabel : function(aLabelHost, aCallback) {
    this._logger.debug("LabelDAO.deleteLabel");

    let query =
      Metacert.QueryHelper.buildQuery(
        Metacert.QueryHelper.TYPE_DELETE, this.TABLE_SCHEMA);

    Metacert.DB.Service.executeAsyncQuery(query, [aLabelHost], aCallback);
  },

  /**
   * Deletes all the labels.
   * @param aCallback the callback method.
   */
  deleteLabels : function(aCallback) {
    this._logger.debug("LabelDAO.deleteLabels");

    var query =
      Metacert.QueryHelper.buildQuery(
        Metacert.QueryHelper.TYPE_CLEAR_TABLE, this.TABLE_SCHEMA);

    Metacert.DB.Service.executeAsyncQuery(query, null, aCallback);
  },


  /**
   * Gets the label with the given host.
   * @param aLabelHost the host of the label to get.
   * @param aCallback the callback method.
   */
  selectLabel : function(aLabelHost, aCallback) {
    this._logger.debug("LabelDAO.selectLabel");

    var query =
      Metacert.QueryHelper.buildQuery(
        Metacert.QueryHelper.TYPE_SELECT, this.TABLE_SCHEMA);
    var localCallback = function(aResult) {
      var labelDTO = null;

      if (aResult && 0 < aResult.length) {
        labelDTO =
          Metacert.ObjectHelper.fromRow(Metacert.LabelDTO.SCHEMA, aResult[0]);
      }
      aCallback(labelDTO);
    };

    Metacert.DB.Service.executeAsyncQuery(query, [aLabelHost], localCallback);
  },

 /**
   * Gets all labels in the database.
   * @param aCallback the callback.
   */
  selectLabels : function(aCallback) {
    this._logger.debug("LabelDAO.selectLabel");

    let query =
      Metacert.QueryHelper.buildQuery(
        Metacert.QueryHelper.TYPE_SELECT_ALL, this.TABLE_SCHEMA);
    let localCallback = function(aResult) {
      let labels = new Array();

      if (aResult && aResult.length > 0) {
        let labelDTO;

        for (let i = 0; i < aResult.length; i++) {
          labelDTO =
            Metacert.ObjectHelper.fromRow(Metacert.LabelDTO.SCHEMA, aResult[i]);
          labels.push(labelDTO);
        }
      }
      aCallback(labels);
    };

    Metacert.DB.Service.executeAsyncQuery(query, null, localCallback);
  }

  ///**
  // * Populates a LabelDTO from a Result.
  // * @param aRow Result object representing the row.
  // * @return populated LabelDTO.
  // */
  //_populateLabelDTO : function(aRow) {
  //  this._logger.trace("LabelDAO._populateLabelDTO");
  //
  //  let LabelDTO =
  //    new Metacert.LabelDTO(
  //      aRow.getValueForColumnName("labelId"),
  //      aRow.getValueForColumnName("labelField"));
  //
  //  return LabelDTO;
  //}
};

/**
 * Constructor.
 */
(function() { this._init(); }).apply(Metacert.LabelDAO);
