/**
 * Copyright (c) 2011 Metacert. All rights reserved.
 */

const EXPORTED_SYMBOLS = [];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;
const Cr = Components.results;

// Firefox URLs that require special treatment.
const RE_FIREFOX_BLANK_PAGE =
  /^jar\:file\:\/\/\/.*\/chrome\/toolkit\.jar\!\/content\/global\/blank\.html$/;

const GOOGLE_SEARCH_RESULTS_URL_REGEXP =
  /^http\:\/\/(www\.){0,1}google(\.\w{2,3}){1,2}/;
const GOOGLE_SEARCH_RESULTS_URL_REGEXP2 =
      /^https\:\/\/(www\.){0,1}(encrypted\.){0,1}google(\.\w{2,3}){1,2}/;

Cu.import("resource://metacert/common/common.js");
Cu.import("resource://metacert/label/labelService.js");
Cu.import("resource://gre/modules/Services.jsm");

/**
 * The Progress Listener.
 */
Metacert.ProgressListener = {
  PREF_METACERT_ENABLED : Metacert.PrefBranch + "blocking.enabled",

  /* Logger for this object. */
  _logger : null,
  /* Windows mediator. */
  _windowMediator : null,
  _stringBundle : null,
  _ioService: null,
  _console: null,
  _blockList: new Array(),

  /**
   * Initializes the resource.
   */
  _init : function() {
    this._logger = Metacert.getLogger("Metacert.ProgressListener");
    this._logger.trace("_init");

    this._windowMediator = Services.wm;

    this._stringBundle = Services.strings.
      createBundle("chrome://metacert/locale/locale.properties");

    let documentLoadService =
      Cc["@mozilla.org/docloaderservice;1"].getService(Ci.nsIWebProgress);
    this._ioService = Services.io;

    this._console = Services.console;

    documentLoadService.addProgressListener(
      this, Ci.nsIWebProgress.NOTIFY_LOCATION);
  },

  /**
   * Called when the location of the window being watched changes. This is not
   * when a load is requested, but rather once it is verified that the load is
   * going to occur in the given window. For instance, a load that starts in a
   * window might send progress and status messages for the new site, but it
   * will not send the onLocationChange until we are sure that we are loading
   * this new page here.
   * @param aWebProgress the nsIWebProgress instance that fired the
   * notification.
   * @param aRequest the associated nsIRequest. This may be null in some cases.
   * @param aLocation the URI of the location that is being loaded.
   */
  onLocationChange : function(aWebProgress, aRequest, aLocation) {
    // XXX: there is no logging here for efficiency reasons.
    //this._logger.trace("_onLocationChange aLocation:" + aLocation.spec);

    try {
      let domWindow = aWebProgress.DOMWindow; // where the URL is loaded.
      let url = aLocation.spec;

      if (this._isMainDOMWindow(domWindow) &&
        this._shouldFilter(aLocation)) {
          let window = this._getChromeWindowForDOMWindow(domWindow);
          let tab = this._getTabForDOMWindow(window, domWindow);

        // First check if the URL should be filtered at all. This prevents the
        // filter operation to happen on the blank page about:blank and chrome
        // URLs. Then see if it's any of the pages that need an automatic redirect
        // on our side.
        if ((null != aRequest) &&
          Metacert.Application.prefs.
            get(this.PREF_METACERT_ENABLED).value == true) {
          let that = this;
          let localCallback = function(aIsOnBlackList, aWebProgress) {
            if (aIsOnBlackList) {
              Metacert.ProgressListener._showBlockDialog(window, tab, aRequest, url);
              if (aWebProgress.DOMWindow.history.length > 1) {
                aWebProgress.DOMWindow.history.back();
              }
            }
          };

          Metacert.LabelService.isOnBlackList(url, localCallback, aWebProgress);
        }

        //Perform Search Annotation
        if (this._isGoogleLocation(url) == true) {
          let appcontent = window.document.getElementById("content");
          let that =this;
          if(appcontent) {
            //Remove the event just in case we are already listening it
            appcontent.removeEventListener("DOMContentLoaded",
              that._onContentLoaded,
            true);

            appcontent.addEventListener("DOMContentLoaded",
              that._onContentLoaded,
            true);
          }

          //Workaround cache pages
          domWindow.setTimeout(function () {
            that._processSearchResultsPage(domWindow.top.document.childNodes[1],
              domWindow);
          }, 1000);
        }

      }
    } catch (e) {
      this._logger.error("onLocationChange: " + e);
      // cancel the request in case of error, to be safe?
      //aRequest.cancel(Ci.nsIRequest.NS_BINDING_ABORTED);
    }
  },

  /**
   * Indicates if the given content and URI should be filtered by the blocking
   * mechanism.
   * @param aContentLocation nsIURI with the location of the content.
   * @return true if the location should be filtered, false otherwise.
   */
  _shouldFilter : function(aContentLocation) {
    this._logger.trace("_shouldFilter");

    let scheme = aContentLocation.scheme;
    let url = aContentLocation.spec;
    //let host = aContentLocation.host;
    let shouldFilter;

    // chrome urls are required to show windows such as the error console.
    // about urls are required to show the blank page and error pages.
    shouldFilter =
      //RE_XXX_DOMAIN.test(host) &&
      ("chrome" != scheme) && ("about:blank" != url) &&
      !RE_FIREFOX_BLANK_PAGE.test(url);

    return shouldFilter;
  },

  /**
   * Indicates if the given window is a sub-frame or the main DOM window in a
   * page.
   * @param aDOMWindow the window to check whether it's the main DOM window or
   * not.
   * @return true if the given DOM window is the main DOM window. false if it's
   * a subframe of a page.
   */
  _isMainDOMWindow : function(aDOMWindow) {
    this._logger.trace("_isMainDOMWindow");

    return (aDOMWindow == aDOMWindow.top);
  },

  /**
   * Gets the chrome window associated with the given DOM HTML window.
   * @param aDOMWindow the nsIDOMWindow that corresponds to the page or frame
   * being loaded.
   * @return the chrome window object that contains the given DOM window object.
   */
  _getChromeWindowForDOMWindow : function(aDOMWindow) {
    // XXX: there is no logging here for efficiency reasons.

    let windows = this._windowMediator.getEnumerator("navigator:browser");
    let window = null;
    let chromeWindow = null;
    let tab = null;

    while (windows.hasMoreElements()) {
      window = windows.getNext();

      tab = this._getTabForDOMWindow(window, aDOMWindow);

      if (null != tab) {
        chromeWindow = window;
        break;
      }
    }

    return chromeWindow;
  },

  /**
   * Gets the tab object associated with the given DOM HTML window.
   * @param aWindow the window where the page or frame is being loaded.
   * @param aDOMWindow the nsIDOMWindow that corresponds to the page or frame
   * being loaded.
   * @return the tab object that contains the given DOM window object.
   */
  _getTabForDOMWindow : function(aWindow, aDOMWindow) {
    // XXX: there is no logging here for efficiency reasons.

    let topDocument = aDOMWindow.top.document;
    let tabIndex = null;
    let tab = null;

    if (aWindow.gBrowser) {
      tabIndex = aWindow.gBrowser.getBrowserIndexForDocument(topDocument);

      if (-1 != tabIndex) {
        tab = aWindow.gBrowser.mTabs[tabIndex];
      }
    }

    return tab;
  },

  _isGoogleLocation : function (aUrl) {
    let result;
    if (aUrl != null) {
      let url = String (aUrl);
      result = url.match(GOOGLE_SEARCH_RESULTS_URL_REGEXP);
      // if the first type fails, try the second one
      if (result == null) {
        result = url.match(GOOGLE_SEARCH_RESULTS_URL_REGEXP2);
      }
    }
    return (result != null);
  },

  /**
   * Called when a new content document is loaded
   */
  _onContentLoaded : function(event) {
    Metacert.ProgressListener._checkContent(event.originalTarget);
    event.currentTarget.addEventListener("DOMNodeInserted",
      Metacert.ProgressListener._onNodeInserted, false);

    try {
      //if (event.originalTarget instanceof HTMLDocument) {
        let win = event.originalTarget.defaultView;
        // XXX: this is the REAL document element that should be processed. For
        // some reason event.originalTarget sometimes doesn't contain the right
        // loaded document, so page processing doesn't work as expected
        let doc = win.top.document;
        let url = String(win.location.href);
        let result = url.match(GOOGLE_SEARCH_RESULTS_URL_REGEXP);
        // if the first type fails, try the second one
        if (result == null) {
          result = url.match(GOOGLE_SEARCH_RESULTS_URL_REGEXP2);
        }

        // add the xxx button only if the loaded page is a search results page
        if (result != null) {
          Metacert.ProgressListener._createPopup(doc, win);
          Metacert.ProgressListener._processSearchResultsPage(doc, win);
        }
   //   }
    } catch (e) {
      Metacert.ProgressListener._logger.error(e);
    }
  },

  _onNodeInserted : function (event) {
    let that = this;
    let content = event.currentTarget;

    if (content == null) {
      return;
    }

    try {
      content.removeEventListener("DOMNodeInserted",
        that._onNodeInserted, false);
    } catch (e) {
      Metacert.ProgressListener._logger.error(e);
    }

    let timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    timer.initWithCallback({
      notify : function() { Metacert.ProgressListener._checkContent(content); }
      }, 1000, Ci.nsITimer.TYPE_ONE_SHOT);

  },

  _checkContent : function (content) {
    let that = this;

    if(content.defaultView != null) {
      if(content.defaultView.frameElement != null) {
        try {
          let url = String(content.defaultView.frameElement.baseURI);
          let result = url.match(GOOGLE_SEARCH_RESULTS_URL_REGEXP);
          // if the first type fails, try the second one
          if (result == null) {
            result = url.match(GOOGLE_SEARCH_RESULTS_URL_REGEXP2);
          }

          // add the stop button only if the loaded page is a search results page
          if (result != null) {
            let doc = content.defaultView.frameElement;
            Metacert.ProgressListener._createPopup(content, content.defaultView);
            Metacert.ProgressListener._processSearchResultsPage(doc, content.defaultView);
          }
        } catch (e) {
          Metacert.ProgressListener._logger.error(e);
        }
      }
    }

    content.addEventListener("DOMNodeInserted", that._onNodeInserted,
      false);
  },

  _showBlockDialog : function(aWindow, aTab, aRequest, aUrl) {
    this._logger.trace("_showBlockDialog");

    if (null != aWindow) {
      aWindow.gBrowser.selectedTab = aTab;
      aWindow.gBrowser.ownerDocument.defaultView.focus();

      aWindow.openDialog(
       "chrome://metacert/content/blockDialog.xul",
        "", "chrome,modal,dialog,centerscreen,resizable=no", aRequest, aUrl);
      try {
        // Cleaning Document, in case it was load
        if (aWindow.gBrowser.contentDocument.defaultView.document.documentElement) {
          aWindow.gBrowser.contentDocument.defaultView.document.documentElement.innerHTML = "";
        }
      } catch (e) {}
    }
  },

  blockPage : function(aRequest) {
    this._logger.trace("blockPage");
    aRequest.cancel(Ci.nsIRequest.NS_BINDING_ABORTED);
  },

  _processSearchResultsPage : function(aDocument, aWindow) {
    Metacert.ProgressListener._logger.trace("_processSearchResultsPage");

    // XPATH expression to get the main anchor of the search results
    const resultAnchorXPathExpression = "//h3[@class='r']/a";
    // XPATH expression to get the main anchor of the IMG results
    const resultAnchorIMGXPathExpression = "//div/span/ul/li/a";

    let loadedDoc = aDocument;
    // use XPATH to process the search results
    let searchResultsLinks =
      Metacert.evaluateXPath(
        loadedDoc, resultAnchorXPathExpression);

    let searchResultsIMGLinks =
      Metacert.evaluateXPath(
        loadedDoc, resultAnchorIMGXPathExpression);

    for (let c = 0; c < searchResultsIMGLinks.length; c++) {
      searchResultsLinks.push(searchResultsIMGLinks[c]);
    }

    let anchor;
    let numLinks = searchResultsLinks.length

    for (let i = 0; i < numLinks; i++) {
      anchor = searchResultsLinks[i];
      let url;
      let urlParameter = "imgurl";
      let regexS = "[\\?&]"+ urlParameter +"=([^&#]*)";
      let regex = new RegExp(regexS);
      let results = regex.exec(anchor.getAttribute("href"));
      if( results == null ){
        url = anchor.getAttribute("href");
      }
      else{
        url = results[1];
      }

      if (anchor.getAttribute("name") != "metacert") {
        anchor.setAttribute("name", "metacert");
        try{
          let localCallback = function(aIsOnBlackList, anchor) {
            if(aIsOnBlackList) {
              // we insert the button next to the search result link
              let h3Element = anchor.parentNode;
              let metacertStop = aWindow.document.createElement("img");

              h3Element.setAttribute ("style", "overflow: visible ! important;")
              h3Element.insertBefore(metacertStop, anchor.nextSibling);
              metacertStop.setAttribute("name", "metacert");
              metacertStop.setAttribute("src",
                "chrome://metacert/skin/images/ae-16.png");
              metacertStop.setAttribute("style", "display:inline;" +
                "padding: 0px; margin-left: 4px; min-width: 16px; width: 16px; " +
                "min-height: 16px; height: 16px; cursor: pointer;");

              let showMetacertPopup = function(aEvent) {
                let content = aEvent.view.document;
                let popup = content.getElementById("metacertPopup");

                popup.setAttribute("style", "position:absolute;z-index:2147483647;" +
                  "clear:both;border:1px solid #000000;top:" + (10 + aEvent.pageY) +
                  "px;left:" + (10 + aEvent.pageX) + "px;display:none;" +
                  "background-color:#FFFFFF;min-height:225px;" +
                  "padding:10px;min-width:160px;max-width:160px;font-size:12px;" +
                  "cursor:default;");
                popup.style.display="block";
              };

              let hideMetacertPopup = function(aEvent) {
                let content = aEvent.view.document;
                let popup = content.getElementById("metacertPopup");

                aWindow.setTimeout(
                  function() {
                    if (popup.getAttribute("show") != "true") {
                      popup.style.display="none";
                    }
                  }, 300);
              };

              let openMetacert = function(aEvent) {
                aWindow.open("https://child-protection.metacert.com/");
              };

              metacertStop.addEventListener("mouseover", showMetacertPopup, false);
              metacertStop.addEventListener("mouseout", hideMetacertPopup, false);
              metacertStop.addEventListener("click", openMetacert, false);
            }
          };
          Metacert.LabelService.isOnBlackList(url, localCallback, anchor);
        } catch(e){
          Metacert.ProgressListener._logger.error(e);
        }
      }
    }
  },

  _createPopup : function(content, aWindow) {
    let body;
    let bodyTags = content.getElementsByTagName("body");
    let aId = "metacertPopup";

    if (bodyTags && bodyTags.length > 0) {
      body = bodyTags[0];
    }

    if(body != null) {
      if (content.getElementById(aId) == null) {
        let popupElement = aWindow.document.createElement("div");
        let style = "position:absolute;display:none;"

        popupElement.setAttribute("id", aId);
        popupElement.setAttribute("style", style);

        let line = aWindow.document.createElement("div");
        line.setAttribute("style", "display:block;min-height:1px;" +
          "max-height:1px;border-top: 1px solid #DCDCDC;");

        let metacertLogo = aWindow.document.createElement("img");
        metacertLogo.setAttribute("src",
          "chrome://metacert/skin/images/metacertPopup.png");
        metacertLogo.setAttribute("style",
          "display:block;min-width:100px;min-height:26px;margin-bottom:5px;");

        let pHeader = aWindow.document.createElement("div");
        pHeader.setAttribute("style",
          "display:block;max-width:160px;font-weight:bold;min-height:50px");

        let h3 = aWindow.document.createElement("h3");
        h3.appendChild(aWindow.document.createTextNode("Adult Entertainment"));
        h3.setAttribute("style", "display:block;" +
          "font-weight:bold;margin-top:10px");
        let imgStop = aWindow.document.createElement("img");
        imgStop.setAttribute("style",
          "display:block;float:right;");
        imgStop.setAttribute("src", "chrome://metacert/skin/images/ae.png");
        pHeader.appendChild(imgStop);
        pHeader.appendChild(h3);

        let p = aWindow.document.createElement("p");
        p.setAttribute("style", "display:block;");
        p.appendChild(aWindow.document.createTextNode(
          "This website has been labelled by MetaCert as it may contain " +
          "adult content. Access to this site will be blocked if the safety " +
          "control has been enabled."));

        let divMoreInfo = aWindow.document.createElement("div");
        divMoreInfo.setAttribute("style", "display:block;");
        divMoreInfo.appendChild(aWindow.document.
          createTextNode("For more information visit "));
        let a = aWindow.document.createElement("a");
        a.setAttribute("href", "https://metacert.com/familysafety");
        a.setAttribute("target", "_blank");
        a.appendChild(aWindow.document.
          createTextNode("https://metacert.com/familysafety"));
        divMoreInfo.appendChild(a);

        let showPopup = function(aEvent) {
          let content = aEvent.view.document;
          let popup = content.getElementById(aId);
          popup.style.display="block";
          popup.setAttribute("show", "true");
          try {
            if (metacertPopupTimer != null){
              aWindow.clearTimeout(metacertPopupTimer);
              metacertPopupTimer = null;
            }
          } catch (e) {}
        };

        let hidePopup = function(aEvent) {
          let content = aEvent.view.document;
          let popup = content.getElementById(aId);

          metacertPopupTimer = aWindow.setTimeout(
            function() {
              popup.setAttribute("show", "false");
              popup.style.display = "none";
            }, 300);
        };

        //manage mouse over event on popup
        popupElement.addEventListener("mouseover", showPopup, false);
        popupElement.addEventListener("mouseout", hidePopup, false);

        popupElement.appendChild(metacertLogo);
        popupElement.appendChild(line);
        popupElement.appendChild(pHeader);
        popupElement.appendChild(p);
        popupElement.appendChild(divMoreInfo);
        body.appendChild(popupElement);
      }
    }
  },

  /**
   * The QueryInterface method provides runtime type discovery.
   * More: http://developer.mozilla.org/en/docs/nsISupports
   * @param aIID the IID of the requested interface.
   * @return the resulting interface pointer.
   */
  QueryInterface : function(aIID) {
    if (aIID.equals(Ci.nsIWebProgressListener) ||
        aIID.equals(Ci.nsISupportsWeakReference) ||
        aIID.equals(Ci.nsISupports)) {
      return this;
    }
    throw Cr.NS_NOINTERFACE;
  }
};

/**
 * Constructor.
 */
(function() {
  this._init();
}).apply(Metacert.ProgressListener);
